@@ -1,6 +1,7 @@
 /* gEDA - GPL Electronic Design Automation
  * libgeda - gEDA's library
- * Copyright (C) 1998-2004 Ales V. Hvezda
+ * Copyright (C) 1998-2010 Ales Hvezda
+ * Copyright (C) 1998-2020 gEDA Contributors (see ChangeLog for details)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -14,7 +15,8 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
+ * MA 02111-1301 USA.
  */
 
 /*! \file globals.h
@@ -30,4 +32,9 @@ extern int logging_dest;
 /* Size of a tab in characters */
 extern int tab_in_chars;
 
+/* Color map for printing */
+extern COLOR print_colors[MAX_COLORS];
+
+extern int page_control_counter;
+
 #endif
