@@ -1,7 +1,7 @@
 ;;; gEDA - GPL Electronic Design Automation
 ;;; gschem - gEDA Schematic Capture
-;;; Copyright (C) 1998-2008 Ales Hvezda
-;;; Copyright (C) 1998-2008 gEDA Contributors (see ChangeLog for details)
+;;; Copyright (C) 1998-2010 Ales Hvezda
+;;; Copyright (C) 1998-2020 gEDA Contributors (see ChangeLog for details)
 ;;;
 ;;; This program is free software; you can redistribute it and/or modify
 ;;; it under the terms of the GNU General Public License as published by
@@ -15,7 +15,11 @@
 ;;;
 ;;; You should have received a copy of the GNU General Public License
 ;;; along with this program; if not, write to the Free Software
-;;; Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+;;; Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
+;;; MA 02111-1301 USA.
+
+(use-modules (ice-9 regex)
+             (geda object))
 
 ;; --------------------------------------------------------------------------
 ;;
@@ -675,6 +679,8 @@
 					       def-alignment-pos)) 
 		      (new-angle (list-ref default-def 
 					   def-angle-pos))
+		      (new-color (list-ref default-def 
+					   def-color-pos))
 		      (new-x (+ (list-ref default-def
 					  def-x-offset-pos)
 				(car ref))) 
@@ -708,12 +714,15 @@
 			      (get-point-of-bound "min-y" new-attrib-bounds))))
 		      )
 		(set-attribute-text-properties! attribute
-						-1 ; keep previous color
+						new-color
 						-1 ; keep previous size
 						new-alignment
 						new-angle
 						(+ new-x x_offset)
 						(+ new-y y_offset))
+		(if (not (= new-color -1)) 
+		    (set-object-color! attribute
+			               new-color))
 		)
 	      )
 	    
