@@ -1,7 +1,7 @@
 ;;; gEDA - GPL Electronic Design Automation
 ;;; gnetlist - gEDA Netlist
-;;; Copyright (C) 1998-2008 Ales Hvezda
-;;; Copyright (C) 1998-2008 gEDA Contributors (see ChangeLog for details)
+;;; Copyright (C) 1998-2010 Ales Hvezda
+;;; Copyright (C) 1998-2020 gEDA Contributors (see ChangeLog for details)
 ;;;
 ;;; This program is free software; you can redistribute it and/or modify
 ;;; it under the terms of the GNU General Public License as published by
@@ -15,10 +15,11 @@
 ;;;
 ;;; You should have received a copy of the GNU General Public License
 ;;; along with this program; if not, write to the Free Software
-;;; Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+;;; Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
+;;; MA 02111-1301 USA.
 
 ;;
-;; SystemC netlist backend written by Jaume Masip 
+;; SystemC netlist backend written by Jaume Masip
 ;; (based on gnet-verilog.scm by Mike Jarabek)
 
 ;; some useful regexes for working with net-names
@@ -29,19 +30,19 @@
 (define numeric  "[0-9]+")
 ;; match on a systemc identifier like:  netname[x:y]
 (define bit-range-reg (make-regexp
-		       (string-append "^(" id-regexp ")[[:space:]]*" 
-				      "\\["
-				      "[[:space:]]*(" numeric ")[[:space:]]*"
-				      ":"
-				      "[[:space:]]*(" numeric ")[[:space:]]*"
-				      "\\]")))
+                       (string-append "^(" id-regexp ")[[:space:]]*"
+                                      "\\["
+                                      "[[:space:]]*(" numeric ")[[:space:]]*"
+                                      ":"
+                                      "[[:space:]]*(" numeric ")[[:space:]]*"
+                                      "\\]")))
 
 ;; match on a systemc identifier like:  netname[x]
-(define single-bit-reg (make-regexp 
-			(string-append "^(" id-regexp ")[[:space:]]*"
-				       "\\["
-				       "[[:space:]]*(" numeric ")[[:space:]]*"
-				       "\\]" )))
+(define single-bit-reg (make-regexp
+                        (string-append "^(" id-regexp ")[[:space:]]*"
+                                       "\\["
+                                       "[[:space:]]*(" numeric ")[[:space:]]*"
+                                       "\\]" )))
 
 ;; match on a systemc identifier like:  netname<type>
 (define systemc-reg (make-regexp
@@ -51,8 +52,8 @@
                                        ">" )))
 
 ;; match on a systemc identifier like:  netname
-(define simple-id-reg (make-regexp 
-		       ( string-append "^(" id-regexp ")$" )))
+(define simple-id-reg (make-regexp
+                       ( string-append "^(" id-regexp ")$" )))
 
 
 ;; return the top level block name for the module
@@ -68,22 +69,22 @@
 ;; This function takes an attribute name, desired value, and a list of
 ;; packages.  For each of the packages, it looks up that attribute, and
 ;; if it matches, that package name is added to the list, and the function
-;; recurses on the remaining packages.  If the attribute does not match, 
+;; recurses on the remaining packages.  If the attribute does not match,
 ;; the function just recuses on the remaing packages. Thanks to Mohina Lal
 ;; for this trick.
 ;;
 
-(define systemc:filter 
+(define systemc:filter
   (lambda (attribute value package-list)
     (cond ((null? package-list) '())
-	  ((string=? (gnetlist:get-package-attribute (car package-list) 
-						      attribute) value)
-	   (cons 
-	    (map (lambda (pin)
-		   (car (gnetlist:get-nets (car package-list) pin)))
-		 (pins (car package-list)))
-	    (systemc:filter attribute value (cdr package-list))))
-	  (else (systemc:filter attribute value (cdr package-list)))))
+          ((string=? (gnetlist:get-package-attribute (car package-list)
+                                                      attribute) value)
+           (cons
+            (map (lambda (pin)
+                   (car (gnetlist:get-nets (car package-list) pin)))
+                 (pins (car package-list)))
+            (systemc:filter attribute value (cdr package-list))))
+          (else (systemc:filter attribute value (cdr package-list)))))
 )
 
 
@@ -93,14 +94,14 @@
 ;; Scan through the list of components, and pins from each one, finding the
 ;; pins that have PINTYPE == CHIPIN, CHIPOUT, CHIPTRI (for inout)
 ;; build three lists one each for the inputs, outputs and inouts
-;; return the a list of three lists that contain the pins in the order 
+;; return the a list of three lists that contain the pins in the order
 ;; we want.
 (define systemc:get-port-list
   (lambda ()
     ;; construct list
     (list (systemc:get-matching-nets "device" "IPAD")
-	  (systemc:get-matching-nets "device" "OPAD")
-	  (systemc:get-matching-nets "device" "IOPAD"))))
+          (systemc:get-matching-nets "device" "OPAD")
+          (systemc:get-matching-nets "device" "IOPAD"))))
 
 ;;
 ;; output the meat of the module port section
@@ -110,10 +111,10 @@
 ;;       PORTNAME , <newline>
 ;;
 (define systemc:write-module-declaration
-  (lambda (module-name port-list p)
+  (lambda (module-name port-list)
     (begin
 
-      (display "#include \"systemc.h\"\n" p) 
+      (display "#include \"systemc.h\"\n")
 
       (for-each (lambda (package)         ; loop on packages
                   (begin
@@ -121,12 +122,12 @@
                       (if (not (memv (string->symbol device) ; ignore specials
                                      (map string->symbol (list "IOPAD" "IPAD" "OPAD" "HIGH" "LOW"))))
                           (begin
-                            (display "#include \"" p)
-                            (systemc:display-escaped-identifier (get-device package) p) 
-                            (display ".h\"\n" p))))))
+                            (display "#include \"")
+                            (systemc:display-escaped-identifier (get-device package))
+                            (display ".h\"\n"))))))
                 packages)
-      (newline p)
-      (display "SC_MODULE (" p) (systemc:display-escaped-identifier module-name p) (display ")\n{\n" p)
+      (newline)
+      (display "SC_MODULE (") (systemc:display-escaped-identifier module-name) (display ")\n{\n")
     )
   )
 )
@@ -134,70 +135,66 @@
 ;; output the module direction section
 ;;
 (define systemc:write-port-directions
-  (lambda (port-list p)
-    (let ((in    (car   port-list))    ; extract list of pins 
-	  (out   (cadr  port-list))
-	  (inout (caddr port-list)))
+  (lambda (port-list)
+    (let ((in    (car   port-list))    ; extract list of pins
+          (out   (cadr  port-list))
+          (inout (caddr port-list)))
       (begin
-	(display "/* Port directions begin here */" p)
-	(newline p)
-	(for-each (lambda (pin)
-		    (begin
-      (display "sc_in<bool> " p)(systemc:display-escaped-identifier (systemc:netname pin) p)(display ";" p)(newline p)
-
-;; (display "sc_in<" p)(display (cadr (cadr pin)) p) (display "> " p) (systemc:display-wire pin p)(display ";" p)(newline p)
-;; (display "  /* " )(display(car pin) ) (display " */ " ) (display(cdr pin))
-;; (display "  /* " )(systemc:display-escaped-identifier (systemc:netname pin) ) (display " */ " ) 
+        (display "/* Port directions begin here */")
+        (newline)
+        (for-each (lambda (pin)
+                    (begin
+                      (display "sc_in<bool> ")
+                      (systemc:display-escaped-identifier (systemc:netname pin))
+                      (display ";")
+                      (newline)
                     )) in)       ; do each input
 
-	(for-each (lambda (pin)
-		    (begin
-		      (display "sc_out<bool> " p)
-		      (systemc:display-escaped-identifier 
-		       (systemc:netname pin) p)
-		      (display ";" p)
-		      (newline p))) out)      ; do each output
+        (for-each (lambda (pin)
+                    (begin
+                      (display "sc_out<bool> ")
+                      (systemc:display-escaped-identifier
+                       (systemc:netname pin))
+                      (display ";")
+                      (newline))) out)      ; do each output
 
-	(for-each (lambda (pin)
-		    (begin
-		      (display "sc_inout<bool> " p)
-		      (systemc:display-escaped-identifier 
-		       (systemc:netname pin) p)
-		      (display ";" p)
-		      (newline p))) inout)    ; do each inout
-		      
-	))))
+        (for-each (lambda (pin)
+                    (begin
+                      (display "sc_inout<bool> ")
+                      (systemc:display-escaped-identifier
+                       (systemc:netname pin))
+                      (display ";")
+                      (newline))) inout)    ; do each inout
+
+        ))))
 ;;
 ;; Top level header
 ;;
 
-(define systemc:write-top-header
-	(lambda (p)
-	  (let ((port-list (systemc:get-port-list)))
-	    (begin
-	      (display "/* structural SystemC generated by gnetlist */\n" p)
-	      (display "/* WARNING: This is a generated file, edits */\n" p)
-	      (display "/*        made here will be lost next time  */\n" p)
-	      (display "/*        you run gnetlist!                 */\n" p)
-	      (display "/* Id ........gnet-systemc.scm (04/09/2003) */\n" p)
-	      (display "/* Source...../home/geda/gnet-systemc.scm   */\n" p)
-	      (display "/* Revision...0.3 (23/09/2003)              */\n" p)
-	      (display "/* Author.....Jaume Masip                   */\n" p)
-	      (newline p)
-	      (systemc:write-module-declaration systemc:get-module-name
-						port-list p)
-	      (newline p)
-	      (systemc:write-port-directions port-list p)
-	      (newline p)))))
+(define (systemc:write-top-header)
+  (let ((port-list (systemc:get-port-list)))
+    (begin
+      (display "/* structural SystemC generated by gnetlist */\n")
+      (display "/* WARNING: This is a generated file, edits */\n")
+      (display "/*        made here will be lost next time  */\n")
+      (display "/*        you run gnetlist!                 */\n")
+      (display "/* Id ........gnet-systemc.scm (04/09/2003) */\n")
+      (display "/* Source...../home/geda/gnet-systemc.scm   */\n")
+      (display "/* Revision...0.3 (23/09/2003)              */\n")
+      (display "/* Author.....Jaume Masip                   */\n")
+      (newline)
+      (systemc:write-module-declaration systemc:get-module-name
+                                        port-list)
+      (newline)
+      (systemc:write-port-directions port-list)
+      (newline))))
 
 ;;
 ;; Footer for file
 ;;
-(define systemc:write-bottom-footer
-  (lambda (p)
-    (display "  }\n};\n" p)
-    (newline p)
-    )
+(define (systemc:write-bottom-footer)
+  (display "  }\n};\n")
+  (newline)
 )
 
 ;;
@@ -211,11 +208,11 @@
 ;;      ))
 (define systemc:net-parse
   (lambda (netname)
-    (let 
-	((bit-range (regexp-exec bit-range-reg netname))
-	 (single-bit (regexp-exec single-bit-reg netname))
-	 (simple-id (regexp-exec simple-id-reg netname))
-	 (systemc   (regexp-exec systemc-reg netname)))
+    (let
+        ((bit-range (regexp-exec bit-range-reg netname))
+         (single-bit (regexp-exec single-bit-reg netname))
+         (simple-id (regexp-exec simple-id-reg netname))
+         (systemc   (regexp-exec systemc-reg netname)))
 
 ;;      (newline)
 ;;      (display "    systemc:net-parse ")
@@ -229,25 +226,25 @@
       (cond
        ;; is it a bit range?
        (bit-range
-	;(display "bit-range" )
-	(list (match:substring bit-range 1) 
-	      (list (string->number (match:substring bit-range 2))
-		    (string->number (match:substring bit-range 3))
-		    (> (string->number (match:substring bit-range 3))
-		       (string->number (match:substring bit-range 2))) 
-		    '#t netname)))
+        ;(display "bit-range" )
+        (list (match:substring bit-range 1)
+              (list (string->number (match:substring bit-range 2))
+                    (string->number (match:substring bit-range 3))
+                    (> (string->number (match:substring bit-range 3))
+                       (string->number (match:substring bit-range 2)))
+                    '#t netname)))
 
        ;; just a single bit?
-       (single-bit 
-	;(display "single-bit")
-	(list (match:substring single-bit 1) 
-	      (list (string->number (match:substring single-bit 2))
-		    (string->number (match:substring single-bit 2))
-		    '#f '#f netname)))
+       (single-bit
+        ;(display "single-bit")
+        (list (match:substring single-bit 1)
+              (list (string->number (match:substring single-bit 2))
+                    (string->number (match:substring single-bit 2))
+                    '#f '#f netname)))
 
        ;; just a systemc signal?
        (systemc
-         (begin 
+         (begin
 ;;            (display "done systemc")(newline)
            (list (match:substring systemc 1)
              (list (string->number (match:substring systemc 2))
@@ -257,16 +254,16 @@
 )
 
        ;; or a net without anything
-       (simple-id   
-	;(display "bare-net")
-	(list (match:substring simple-id 1) (list 0 0 #f #f netname)))
+       (simple-id
+        ;(display "bare-net")
+        (list (match:substring simple-id 1) (list 0 0 #f #f netname)))
 
-       (else       
-	(display 
-	 (string-append "Warning: `" netname 
-			"' is not likely a valid Verilog identifier"))
-	(newline)
-	(list netname (list 0 0 #f #f netname)))
+       (else
+        (display
+         (string-append "Warning: `" netname
+                        "' is not likely a valid Verilog identifier"))
+        (newline)
+        (list netname (list 0 0 #f #f netname)))
        )))
 )
 
@@ -276,11 +273,11 @@
 ;;
 (define systemc:identifier?
   (lambda (netname)
-    (let 
-	((bit-range (regexp-exec bit-range-reg netname))
-	 (single-bit (regexp-exec single-bit-reg netname))
-	 (simple-id (regexp-exec simple-id-reg netname))
-	 (systemc (regexp-exec systemc-reg netname)))
+    (let
+        ((bit-range (regexp-exec bit-range-reg netname))
+         (single-bit (regexp-exec single-bit-reg netname))
+         (simple-id (regexp-exec simple-id-reg netname))
+         (systemc (regexp-exec systemc-reg netname)))
 
       ;; check over each expression type, return
       ;; result
@@ -294,15 +291,14 @@
        ))))
 
 ;;
-;; Display a systemc identifier that is escaped if needed 
+;; Display a systemc identifier that is escaped if needed
 ;;
 (define systemc:display-escaped-identifier
-  (lambda (netname port)
+  (lambda (netname)
     (if (systemc:identifier? netname)
-	(display netname port) ; just display the identifier
-	;;(display (string-append "\\" netname " ") port)))) ; need to escape
-	(display netname port)))) ; need to escape
-    
+        (display netname) ; just display the identifier
+        (display netname)))) ; need to escape
+
 
 ;;
 ;; return just the netname part of a systemc identifier
@@ -310,10 +306,10 @@
 (define systemc:netname
   (lambda (netname)
     (car (systemc:net-parse netname))))
-      
+
 ;;  Update the given bit range with data passed.  Take care
 ;;  of ordering issues.
-;;  
+;;
 ;;   n1     : new first range
 ;;   n2     : new second range
 ;;   old-n1 : first range to be updated
@@ -322,22 +318,22 @@
 (define systemc:update-range
   (lambda (n1 n2 old-n1 old-n2 increasing)
     (let ((rn1 (if increasing
-		   (min n1 old-n1)     ; originally increasing
-		   (max n1 old-n1)))   ; originally decreasing
+                   (min n1 old-n1)     ; originally increasing
+                   (max n1 old-n1)))   ; originally decreasing
 
-	  (rn2 (if increasing
-		   (max n2 old-n2)     ; originally increasing
-		   (min n2 old-n2))))
-;      (display (string-append "increasing:" 
-;			      (if increasing "increasing" "decreasing")
-;			      " rn1:" (number->string rn1) 
-;			      " rn2:" (number->string rn2)
-;			      " n1:" (number->string n1)
-;			      " n2:" (number->string n2)
-;			      " old-n1:" (number->string old-n1)
-;			      " old-n2:" (number->string old-n2))) (newline)
+          (rn2 (if increasing
+                   (max n2 old-n2)     ; originally increasing
+                   (min n2 old-n2))))
+;      (display (string-append "increasing:"
+;                             (if increasing "increasing" "decreasing")
+;                             " rn1:" (number->string rn1)
+;                             " rn2:" (number->string rn2)
+;                             " n1:" (number->string n1)
+;                             " n2:" (number->string n2)
+;                             " old-n1:" (number->string old-n1)
+;                             " old-n2:" (number->string old-n2))) (newline)
       (list rn1 rn2)
-	
+
       )))
 
 
@@ -345,19 +341,19 @@
 ;; parameters
 (define systemc:update-record
   (lambda (n1
-	   n2 
-	   list-n1 
-	   list-n2
-	   increasing
-	   sure
-	   real)
+           n2
+           list-n1
+           list-n2
+           increasing
+           sure
+           real)
     (list
-     (append (systemc:update-range 
-	      n1 n2 list-n1 list-n2
-	      increasing)
-	     (list increasing
-		   sure
-		   real)))))
+     (append (systemc:update-range
+              n1 n2 list-n1 list-n2
+              increasing)
+             (list increasing
+                   sure
+                   real)))))
 
 ;;
 ;;  Work over the list of `unique' nets in the design,
@@ -480,22 +476,15 @@
 ;;
 ;;  Display a net in a legal systemc format, based on the object passed
 (define systemc:display-wire
-  (lambda (wire p)
+  (lambda (wire)
     ;; figure out if we need a bit range
     (let ((name            (car wire))
-	  (n1              (car (cadr wire)))
-	  (n2              (cadr (cadr wire)))
+          (n1              (car (cadr wire)))
+          (n2              (cadr (cadr wire)))
           (increasing      (caddr (cadr wire)))
-	  )
-      
-;;      (if (not (and (zero? n1) (zero? n2)))
-;;	  (begin     ;; yes, print it
-;;	    (display "[ " p)(display n1 p)(display " : " p)(display n2 p)(display " ] " p) ) )
-    ;; print the wire name
-      (systemc:display-escaped-identifier name p)
-      ;;(systemc:display-escaped-identifier n1 p)
-      ;;(systemc:display-escaped-identifier n2 p)
-      ;;(systemc:display-escaped-identifier increasing p)
+          )
+
+      (systemc:display-escaped-identifier name)
     )
   )
 )
@@ -503,58 +492,51 @@
 ;;
 ;;  Loop over the list of nets in the design, writing one by one
 ;;
-(define systemc:write-wires
-  (lambda (p)
-    (display "/* Wires from the design */" p)
-    (newline p)
-    (for-each (lambda (wire)          ; print a wire statement for each
-    ;;            (let ((name (car wire)) (n1 (car (cadr wire))) (n2 (cadr (cadr wire))) (increasing (caddr (cadr wire)))))
-;;    (display "/* Wires from the design */")(newline)
-;;                (display "systemc:write-wires -> ")(display wire)(newline)
-		(display "sc_signal<" p)
-                (display (cadr (cadr wire)) p)        
-		(display "> " p)
-		(systemc:display-wire wire p)
-		(display ";" p)
-		(newline p))
-	      systemc:get-nets ) 
-    (newline p)))
+(define (systemc:write-wires)
+  (display "/* Wires from the design */")
+  (newline)
+  (for-each (lambda (wire)          ; print a wire statement for each
+              (display "sc_signal<")
+              (display (cadr (cadr wire)))
+              (display "> ")
+              (systemc:display-wire wire)
+              (display ";")
+              (newline))
+            systemc:get-nets )
+  (newline))
 
 ;;
 ;;  Output any continuous assignment statements generated
-;; by placing `high' and `low' components on the board 
+;; by placing `high' and `low' components on the board
 ;;
-(define systemc:write-continuous-assigns
-  (lambda (p)
-;;    (display "/* continuous assignments */" p) (newline p)
-    (for-each (lambda (wire)             ; do high values
-		(begin
-		  (display "assign " p) 	
-		  ;; XXX fixme, multiple bit widths!
-		  (systemc:display-escaped-identifier wire p) 
-		  (display " = 1'b1;" p) 
-		  (newline p)))
-	      (systemc:get-matching-nets "device" "HIGH"))
+(define (systemc:write-continuous-assigns)
+  (for-each (lambda (wire)             ; do high values
+              (begin
+                (display "assign ")
+                ;; XXX fixme, multiple bit widths!
+                (systemc:display-escaped-identifier wire)
+                (display " = 1'b1;")
+                (newline)))
+            (systemc:get-matching-nets "device" "HIGH"))
 
-    (for-each (lambda (wire)
-		(begin
-		  (display "assign " p) 
-		  ;; XXX fixme, multiple bit widths!
-		  (systemc:display-escaped-identifier wire p)
-		  (display " = 1'b0;" p)
-		  (newline p)))
-	      (systemc:get-matching-nets "device" "LOW"))
-    (newline p))
-)
+  (for-each (lambda (wire)
+              (begin
+                (display "assign ")
+                ;; XXX fixme, multiple bit widths!
+                (systemc:display-escaped-identifier wire)
+                (display " = 1'b0;")
+                (newline)))
+            (systemc:get-matching-nets "device" "LOW"))
+  (newline))
 
 
 
 ;;
-;; Top level component writing 
+;; Top level component writing
 ;;
 ;; Output a compoment instatantiation for each of the
 ;; components on the board
-;; 
+;;
 ;; use the format:
 ;;
 ;;  device-attribute refdes (
@@ -566,10 +548,10 @@
 (define c_p #f)
 
 (define systemc:components
-  (lambda (packages port)
+  (lambda (packages)
     (begin
       (set! c_p #f)
-      (display "/* Package instantiations */" port) (newline port)
+      (display "/* Package instantiations */") (newline)
 
       (for-each (lambda (package)         ; loop on packages
                   (begin
@@ -577,14 +559,14 @@
                       (if (not (memv (string->symbol device) ; ignore specials
                                      (map string->symbol (list "IOPAD" "IPAD" "OPAD" "HIGH" "LOW"))))
                           (begin
-                            (systemc:display-escaped-identifier (get-device package) port) (display " " port)
-                            (systemc:display-escaped-identifier package port) (display ";" port)
-                            (newline port))))))
+                            (systemc:display-escaped-identifier (get-device package)) (display " ")
+                            (systemc:display-escaped-identifier package) (display ";")
+                            (newline))))))
                 packages)
 
-      (newline port)
-      (display "SC_CTOR(" port) (systemc:display-escaped-identifier systemc:get-module-name port) 
-      (display "):\n" port) 
+      (newline)
+      (display "SC_CTOR(") (systemc:display-escaped-identifier systemc:get-module-name)
+      (display "):\n")
 
       (for-each (lambda (package)         ; loop on packages
                   (begin
@@ -592,62 +574,60 @@
                       (if (not (memv (string->symbol device) ; ignore specials
                                      (map string->symbol (list "IOPAD" "IPAD" "OPAD" "HIGH" "LOW"))))
                           (begin
-                            (if c_p (begin (display "," port) (newline port)) (set! c_p #t))
-                            (display "    " port)
-                            (systemc:display-escaped-identifier package port) 
-                            (display "(\"" port)
-                            (systemc:display-escaped-identifier package port)
+                            (if c_p (begin (display ",") (newline)) (set! c_p #t))
+                            (display "    ")
+                            (systemc:display-escaped-identifier package)
+                            (display "(\"")
+                            (systemc:display-escaped-identifier package)
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
-;;(do ((i n (- i 1))) ((zero? i)) (move-n-turn (/ 360 n)))
-(do ((lp 1 (+ lp 1))) ((> lp 32)) 
-;;  (begin (display lp)(newline)))
+(do ((lp 1 (+ lp 1))) ((> lp 32))
   (let* ((attr (string-append "attr" (number->string lp)))
        (description (gnetlist:get-package-attribute package attr)))
       (begin
           (if (not (string=? description "unknown"))
-               (begin (display "\",\"" port) (display description port)))))
-) 
+               (begin (display "\",\"") (display description)))))
+)
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
-                             (display "\")" port)
+                             (display "\")")
                             )))))
                 packages)
-      (display "\n  {" port) 
+      (display "\n  {")
 
       (for-each (lambda (package)         ; loop on packages
-		  (begin
-		    (let ((device (get-device package)))
- 		      (if (not (memv (string->symbol device) ; ignore specials
- 				     (map string->symbol (list "IOPAD" "IPAD" "OPAD" "HIGH" "LOW"))))
- 			  (begin
-			    ; if this module wants positional pins, 
-			    ; then output that format, otherwise
-			    ; output normal named declaration
-			    (systemc:display-connections package 
-			     (string=? (gnetlist:get-package-attribute package "VERILOG_PORTS" ) "POSITIONAL") port)
- 			    )))))
- 		packages))))
+                  (begin
+                    (let ((device (get-device package)))
+                      (if (not (memv (string->symbol device) ; ignore specials
+                                     (map string->symbol (list "IOPAD" "IPAD" "OPAD" "HIGH" "LOW"))))
+                          (begin
+                            ; if this module wants positional pins,
+                            ; then output that format, otherwise
+                            ; output normal named declaration
+                            (systemc:display-connections package
+                             (string=? (gnetlist:get-package-attribute package "VERILOG_PORTS" ) "POSITIONAL"))
+                            )))))
+                packages))))
 
 ;;
 ;; output a module connection for the package given to us with named ports
 ;;
 (define systemc:display-connections
-   (lambda (package positional port)
+   (lambda (package positional)
      (begin
-       (let ( (pin-list (gnetlist:get-pins-nets package)) 
-	      (comma_pending #f) )
- 	(if (not (null? pin-list))
- 	    (begin
-	      (newline port)
- 	      (for-each (lambda (pin)
- 			  (if (not  (strncmp? (cdr pin) "unconnected_pin" 15) )
-			      (begin 
- 			        (display "    " port)(systemc:display-escaped-identifier package port) 
-				(systemc:display-pin pin positional port)
-                                (display ";" port) (newline port))))
- 			pin-list)
- 	      )))))
+       (let ( (pin-list (gnetlist:get-pins-nets package))
+              (comma_pending #f) )
+        (if (not (null? pin-list))
+            (begin
+              (newline)
+              (for-each (lambda (pin)
+                          (if (not  (strncmp? (cdr pin) "unconnected_pin" 15) )
+                              (begin
+                                (display "    ")(systemc:display-escaped-identifier package)
+                                (systemc:display-pin pin positional)
+                                (display ";") (newline))))
+                        pin-list)
+              )))))
 )
 
 ;;
@@ -657,50 +637,45 @@
 ;;    /* PINNAME */ NETNAME
 ;;
 ;;  otherwise emit:
-;; 
+;;
 ;;      .PINNAME ( NETNAME )
 ;;
 (define systemc:display-pin
-    (lambda (pin positional port)
+    (lambda (pin positional)
       (let
           ((systemc (regexp-exec systemc-reg (cdr pin))))
           (begin
             (if positional
-                (begin    ; output a positional port instanace
-                  (display "  /* " port)
-                  (display (car pin) port)  ; add in name for debugging
-                  (display " */ " port )
-                  (display (cdr pin) port))
+                (begin    ; output a positional port instance
+                  (display "  /* ")
+                  (display (car pin))  ; add in name for debugging
+                  (display " */ " )
+                  (display (cdr pin)))
                 (begin    ; else output a named port instance
-                  (display "." port)
+                  (display ".")
                   ; Display the escaped version of the identifier
-                  (systemc:display-escaped-identifier (car pin) port)
-                  (display "(" port)
+                  (systemc:display-escaped-identifier (car pin))
+                  (display "(")
                   (if systemc
-                    (display (match:substring systemc 1) port)
-                    (systemc:display-escaped-identifier (cdr pin) port))
-                  (display ")" port)))))))
-    
-	 
+                    (display (match:substring systemc 1))
+                    (systemc:display-escaped-identifier (cdr pin)))
+                  (display ")")))))))
+
+
 
 ;;; Highest level function
 ;;; Write Structural systemc representation of the schematic
 ;;;
-(define systemc 
-  (lambda (output-filename)
-    (let ((port (open-output-file output-filename)))
-      (begin
-        (systemc:get-nets-once!)
-	(systemc:write-top-header port)
-;;        (display "***** start write-wires ********")(newline)
-	(systemc:write-wires port)
-;;        (display "***** end write-wires ********")(newline)
-	(systemc:write-continuous-assigns port)
-	(systemc:components packages port)
-	(systemc:write-bottom-footer port)
-	)
-      (close-output-port port)
-      )
+(define (systemc output-filename)
+  (set-current-output-port (gnetlist:output-port output-filename))
+  (begin
+    (systemc:get-nets-once!)
+    (systemc:write-top-header)
+;;    (display "***** start write-wires ********")(newline)
+    (systemc:write-wires)
+;;    (display "***** end write-wires ********")(newline)
+    (systemc:write-continuous-assigns)
+    (systemc:components packages)
+    (systemc:write-bottom-footer)
     )
-) 
-
+  (close-output-port (current-output-port)))
